@@ -77,6 +77,28 @@ public interface Canvas {
      */
     Dimension getDimension();
 
+    /**
+     * Anti-aliasing level getter.
+     *  - 0 for 1x
+     *  - 1 for 2x
+     *  - 2 for 4x
+     *  - 3 for 8x
+     *  - 4 for 16x
+     * @return the anti-aliasing level.
+     */
+    public int getAntiAliasingLevel();
+
+    /**
+     * Anti-aliasing level setter.
+     *  - 0 for 1x
+     *  - 1 for 2x
+     *  - 2 for 4x
+     *  - 3 for 8x
+     *  - 4 for 16x
+     * @param antiAliasingLevel the new level.
+     */
+    public void setAntiAliasingLevel(int antiAliasingLevel);
+
     /** Ask the canvas to perform asynchronous drawing. */
     void redraw();
 
