@@ -39,11 +39,16 @@ public final class G2DCanvas implements Canvas {
     private final Motor3D motor;
     private final Dimension dimension;
 
+    private boolean drawEnabled = true;
+
+    /** The anti-aliasing level */
+    private int antiAliasingLevel = 0;
+
     private static final PickingManager PICKINGMANAGER = new PickingManager() {
-	    
-	    @Override
-	    public void addPickingTask(PickingTask pickingTask) { }
-	};
+
+        @Override
+        public void addPickingTask(PickingTask pickingTask) { }
+    };
 
     /**
      * The current mainDrawer.
@@ -55,18 +60,18 @@ public final class G2DCanvas implements Canvas {
      * @param autoDrawable the G2D autoDrawable this canvas depend on.
      */
     G2DCanvas(Graphics2D g2d, int width, int height) {
-	this.dimension = new Dimension(width, height);
-	this.motor = new Motor3D(this, g2d, dimension);
+        this.dimension = new Dimension(width, height);
+        this.motor = new Motor3D(this, g2d, dimension);
 
         buffersManager = new G2DBuffersManager();
         rendererManager = new G2DRendererManager();
         drawingTools = new G2DDrawingTools(this);
-	motor.setClippingPlanes(drawingTools.getClippingManager().getClippingPlanes());
+        motor.setClippingPlanes(drawingTools.getClippingManager().getClippingPlanes());
         textureManager = new G2DTextureManager(this);
     }
 
     public DrawingTools getDrawingTools() {
-	return drawingTools;
+        return drawingTools;
     }
 
     // Implementation of getter & setter from Canvas.
@@ -93,12 +98,12 @@ public final class G2DCanvas implements Canvas {
 
     @Override
     public PickingManager getPickingManager() {
-    	return PICKINGMANAGER;
+        return PICKINGMANAGER;
     }
 
     @Override
     public G2DTextureManager getTextureManager() {
-	return textureManager;
+        return textureManager;
     }
 
     @Override
@@ -116,9 +121,19 @@ public final class G2DCanvas implements Canvas {
         return dimension;
     }
 
+    @Override
+    public int getAntiAliasingLevel() {
+        return antiAliasingLevel;
+    }
+
+    @Override
+    public void setAntiAliasingLevel(int antiAliasingLevel) {
+        this.antiAliasingLevel = antiAliasingLevel;
+    }
+
     @Override
     public void redraw() {
-	    draw();
+        draw();
     }
 
     @Override
@@ -127,14 +142,25 @@ public final class G2DCanvas implements Canvas {
 
     // G2DCanvas specific getter.
 
+    public void disableDraw() {
+        drawEnabled = false;
+    }
+
+    public void enableDraw() {
+        drawEnabled = true;
+    }
+
     public void draw() {
-	try {
-	    mainDrawer.draw(drawingTools);
-	    getMotor3D().draw();
-	} catch (Exception e) {
-	    System.out.println(e+"::::"+mainDrawer);
-	    e.printStackTrace();
-	}
+        if (drawEnabled) {
+            try {
+                mainDrawer.draw(drawingTools);
+                getMotor3D().setAntialiased(antiAliasingLevel != 0);
+                getMotor3D().draw();
+            } catch (Exception e) {
+                System.out.println(e + "::::" + mainDrawer);
+                e.printStackTrace();
+            }
+        }
     }
 
     /**
@@ -148,6 +174,6 @@ public final class G2DCanvas implements Canvas {
     @Override
     public void destroy() {
         // TODO Auto-generated method stub
-        
+
     }
 }
