@@ -11,10 +11,13 @@
 
 package org.scilab.forge.scirenderer.implementation.g2d.motor;
 
+import java.awt.BasicStroke;
 import java.awt.Color;
 import java.awt.Graphics2D;
 import java.awt.RenderingHints;
 import java.awt.Shape;
+import java.awt.Stroke;
+import java.awt.geom.Area;
 import java.awt.geom.Path2D;
 import java.awt.image.BufferedImage;
 import java.util.ArrayList;
@@ -32,12 +35,7 @@ import org.scilab.forge.scirenderer.tranformations.Vector4d;
 public class Triangle extends ConvexObject {
 
     private static final Color[] COLORS = new Color[]{Color.BLACK, Color.BLACK, Color.BLACK};
-    private static Class IMAGEGRAPHICS2DCLASS;
-
-    static {
-	BufferedImage i = new BufferedImage(1, 1, BufferedImage.TYPE_INT_ARGB);
-	IMAGEGRAPHICS2DCLASS = i.createGraphics().getClass();
-    }
+    private static final Stroke stroke = new BasicStroke(1f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL);
 
     private SpritedRectangle sprite;
     private BufferedImage image;
@@ -73,13 +71,21 @@ public class Triangle extends ConvexObject {
         return super.isBehind(o);
     }
 
+    public boolean isIn2D() {
+        return vertices[0].getZ() == 0 && vertices[1].getZ() == 0 && vertices[2].getZ() == 0;
+    }
+
     public boolean addSegment(Segment s) {
         if (isSegmentInside(s)) {
             if (segments == null) {
                 segments = new TreeSet<Segment>();
             }
+            if (segments.contains(s)) {
+                segments.remove(s);
+            }
             segments.add(s);
             s.addConvexObject(this);
+
             return true;
         }
 
@@ -99,6 +105,9 @@ public class Triangle extends ConvexObject {
     public void replaceSegment(Segment s, List<Segment> segs) {
         segments.remove(s);
         for (Segment ss : segs) {
+            if (segments.contains(ss)) {
+                segments.remove(ss);
+            }
             segments.add(ss);
             ss.addConvexObject(this);
         }
@@ -173,11 +182,12 @@ public class Triangle extends ConvexObject {
             DrawTools.drawTriangleTexture(g2d, image, new double[]{textureCoords[0].getX(), textureCoords[1].getX(), textureCoords[2].getX()}, new double[]{textureCoords[0].getY(), textureCoords[1].getY(), textureCoords[2].getY()}, new double[]{vertices[0].getX(), vertices[1].getX(), vertices[2].getX()}, new double[]{vertices[0].getY(), vertices[1].getY(), vertices[2].getY()}, key);
         } else if (colors[0].equals(colors[1]) && colors[1].equals(colors[2])) {
             Path2D contour = getProjectedContour();
+            Area area = new Area(contour);
+            // Trick to paint the triangle and its outline
+            area.add(new Area(stroke.createStrokedShape(contour)));
+
             g2d.setColor(colors[0]);
-            g2d.fill(contour);
-	    if (g2d.getClass() == IMAGEGRAPHICS2DCLASS) {
-		g2d.draw(contour);
-	    }
+            g2d.fill(area);
         } else {
             DrawTools.fillGouraud(g2d, this);
         }
@@ -277,7 +287,7 @@ public class Triangle extends ConvexObject {
             return true;
         }
 
-        return isPointInside(s.vertices[0], false) && isPointInside(s.vertices[1], false);
+        return isPointInside(s.vertices[0], false) || isPointInside(s.vertices[1], false) || check2DIntersection(s);
     }
 
     protected boolean isSegmentIntersects(final Segment s) {
@@ -400,6 +410,14 @@ public class Triangle extends ConvexObject {
                 Color[] colors1 = new Color[]{t.colors[i], t.colors[j], cb};
                 Color[] colors2 = new Color[]{t.colors[i], cb, t.colors[k]};
 
+                Vector3d[] tvertices1 = null;
+                Vector3d[] tvertices2 = null;
+                if (t.textureCoords != null) {
+                    Vector3d tvb = Vector3d.getBarycenter(t.textureCoords[j], t.textureCoords[k], weight, 1 - weight);
+                    tvertices1 = new Vector3d[]{t.textureCoords[i], t.textureCoords[j], tvb};
+                    tvertices2 = new Vector3d[]{t.textureCoords[i], tvb, t.textureCoords[k]};
+                }
+
                 try {
                     Triangle t1 = new Triangle(vertices1, colors1, null);
                     t1.setSprite(t.getSprite());
@@ -407,6 +425,12 @@ public class Triangle extends ConvexObject {
                     Triangle t2 = new Triangle(vertices2, colors2, null);
                     t2.setSprite(t.getSprite());
                     list.add(t2);
+                    if (t.textureCoords != null) {
+                        t1.textureCoords = tvertices1;
+                        t2.textureCoords = tvertices2;
+                        t1.image = t2.image = t.image;
+                        t1.filter = t2.filter = t.filter;
+                    }
                 } catch (InvalidPolygonException e) { }
 
                 addSegments(list, t, p, Vector3d.product(t.v0v1, n), n);
@@ -420,6 +444,7 @@ public class Triangle extends ConvexObject {
 
         Color cu, cv;
         Vector3d u, v;
+        Vector3d tu = null, tv = null;
         // t has not been broken, so continue...
         if (a < 0 || a > 1) {
             i = 2;
@@ -429,6 +454,10 @@ public class Triangle extends ConvexObject {
             v = Vector3d.getBarycenter(t.vertices[0], t.vertices[2], b, 1 - b);
             cu = getColorsBarycenter(t.colors[1], t.colors[2], c, 1 - c);
             cv = getColorsBarycenter(t.colors[0], t.colors[2], b, 1 - b);
+            if (t.textureCoords != null) {
+                tu = Vector3d.getBarycenter(t.textureCoords[1], t.textureCoords[2], c, 1 - c);
+                tv = Vector3d.getBarycenter(t.textureCoords[0], t.textureCoords[2], b, 1 - b);
+            }
         } else if (b < 0 || b > 1) {
             i = 1;
             j = 2;
@@ -437,6 +466,10 @@ public class Triangle extends ConvexObject {
             v = Vector3d.getBarycenter(t.vertices[1], t.vertices[2], c, 1 - c);
             cu = getColorsBarycenter(t.colors[0], t.colors[1], a, 1 - a);
             cv = getColorsBarycenter(t.colors[1], t.colors[2], c, 1 - c);
+            if (t.textureCoords != null) {
+                tu = Vector3d.getBarycenter(t.textureCoords[0], t.textureCoords[1], a, 1 - a);
+                tv = Vector3d.getBarycenter(t.textureCoords[1], t.textureCoords[2], c, 1 - c);
+            }
         } else {
             i = 0;
             j = 1;
@@ -445,6 +478,10 @@ public class Triangle extends ConvexObject {
             v = Vector3d.getBarycenter(t.vertices[0], t.vertices[1], a, 1 - a);
             cu = getColorsBarycenter(t.colors[0], t.colors[2], b, 1 - b);
             cv = getColorsBarycenter(t.colors[0], t.colors[1], a, 1 - a);
+            if (t.textureCoords != null) {
+                tu = Vector3d.getBarycenter(t.textureCoords[0], t.textureCoords[2], b, 1 - b);
+                tv = Vector3d.getBarycenter(t.textureCoords[0], t.textureCoords[1], a, 1 - a);
+            }
         }
 
         Vector3d[] vertices1 = new Vector3d[]{u, t.vertices[i], v};
@@ -454,6 +491,15 @@ public class Triangle extends ConvexObject {
         Vector3d[] vertices3 = new Vector3d[]{u, t.vertices[j], t.vertices[k]};
         Color[] colors3 = new Color[]{cu, t.colors[j], t.colors[k]};
 
+        Vector3d[] tvertices1 = null;
+        Vector3d[] tvertices2 = null;
+        Vector3d[] tvertices3 = null;
+        if (t.textureCoords != null) {
+            tvertices1 = new Vector3d[]{tu, t.textureCoords[i], tv};
+            tvertices2 = new Vector3d[]{tu, tv, t.textureCoords[j]};
+            tvertices3 = new Vector3d[]{tu, t.textureCoords[j], t.textureCoords[k]};
+        }
+
         try {
             Triangle t1 = new Triangle(vertices1, colors1, null);
             t1.setSprite(t.getSprite());
@@ -464,6 +510,13 @@ public class Triangle extends ConvexObject {
             Triangle t3 = new Triangle(vertices3, colors3, null);
             t3.setSprite(t.getSprite());
             list.add(t3);
+            if (t.textureCoords != null) {
+                t1.textureCoords = tvertices1;
+                t2.textureCoords = tvertices2;
+                t3.textureCoords = tvertices3;
+                t1.image = t2.image = t3.image = t.image;
+                t1.filter = t2.filter = t3.filter = t.filter;
+            }
         } catch (InvalidPolygonException e) { }
 
         addSegments(list, t, p, Vector3d.product(t.v0v1, n), n);
@@ -473,7 +526,6 @@ public class Triangle extends ConvexObject {
 
     private static final void addSegments(List<ConvexObject> list, Triangle t, Vector3d p, Vector3d u, Vector3d n) {
         if (t.segments != null) {
-            boolean bb = false;
             List<Segment> segs = new ArrayList<Segment>();
             for (Segment s : t.segments) {
                 s.removeConvexObject(t);
@@ -483,6 +535,7 @@ public class Triangle extends ConvexObject {
                     s.replaceSegment(l);
                 }
             }
+            t.textureCoords = null;
 
             for (Segment s : segs) {
                 for (ConvexObject tri : list) {
