@@ -30,6 +30,7 @@ import javax.media.opengl.GLCapabilities;
 import javax.media.opengl.GLContext;
 import javax.media.opengl.GLDrawableFactory;
 import javax.media.opengl.GLEventListener;
+import javax.media.opengl.GLException;
 import javax.media.opengl.GLPbuffer;
 import javax.media.opengl.GLProfile;
 
@@ -44,17 +45,17 @@ public final class JoGLCanvas implements Canvas, GLEventListener {
         {{0.0, 0.0}},
         {{0.5, 0.5}, {0.5, -0.5}},
         {
-            {-0.25, -0.5}, {0.25, 0.5}, {0.75, -0.5}, {0.25, 0.5}
+            { -0.25, -0.5}, {0.25, 0.5}, {0.75, -0.5}, {0.25, 0.5}
         },
         {
-            {0.125, -0.125}, {-0.875, 0.875}, {-0.375, 0.375}, {0.375, 0.625},
-            {0.625, -0.625}, {0.875, 0.125}, {-0.125, -0.875}, {-0.625, -0.375}
+            {0.125, -0.125}, { -0.875, 0.875}, { -0.375, 0.375}, {0.375, 0.625},
+            {0.625, -0.625}, {0.875, 0.125}, { -0.125, -0.875}, { -0.625, -0.375}
         },
         {
-            {-0.25, -0.125}, {0.25, -0.875}, {0.75, -0.625}, {-0.75, -0.875},
-            {-0.25, 0.375}, {0.75, -0.125}, {0.25, 0.125}, {-0.25, 0.875},
-            {0.25, -0.375}, {-0.75, 0.125}, {-0.75, 0.625}, {-0.25, -0.625},
-            {0.75, 0.875}, {0.75, 0.375}, {-0.75, -0.375}, {0.25, 0.625}
+            { -0.25, -0.125}, {0.25, -0.875}, {0.75, -0.625}, { -0.75, -0.875},
+            { -0.25, 0.375}, {0.75, -0.125}, {0.25, 0.125}, { -0.25, 0.875},
+            {0.25, -0.375}, { -0.75, 0.125}, { -0.75, 0.625}, { -0.25, -0.625},
+            {0.75, 0.875}, {0.75, 0.375}, { -0.75, -0.375}, {0.25, 0.625}
         }
     };
 
@@ -171,34 +172,18 @@ public final class JoGLCanvas implements Canvas, GLEventListener {
         canvasAnimator.waitEndOfDrawing();
     }
 
-    // JoGLCanvas specific getter.
-
-    /**
-     * Anti-aliasing level getter.
-     *  - 0 for 1x
-     *  - 1 for 2x
-     *  - 2 for 4x
-     *  - 3 for 8x
-     *  - 4 for 16x
-     * @return the anti-aliasing level.
-     */
+    @Override
     public int getAntiAliasingLevel() {
         return antiAliasingLevel;
     }
 
-    /**
-     * Anti-aliasing level setter.
-     *  - 0 for 1x
-     *  - 1 for 2x
-     *  - 2 for 4x
-     *  - 3 for 8x
-     *  - 4 for 16x
-     * @param antiAliasingLevel the new level.
-     */
+    @Override
     public void setAntiAliasingLevel(int antiAliasingLevel) {
         this.antiAliasingLevel = antiAliasingLevel;
     }
 
+    // JoGLCanvas specific getter.
+
     /**
      * Return the OpenGl context.
      * @return the OpenGl context.
@@ -250,7 +235,7 @@ public final class JoGLCanvas implements Canvas, GLEventListener {
             ((GLPbuffer) autoDrawable).destroy();
         }
         try {
-            canvasAnimator.finalize();
+            canvasAnimator.finalize();//Thread.dumpStack();
         } catch (Throwable e) {
             // TODO: handle exception
         }
@@ -283,6 +268,9 @@ public final class JoGLCanvas implements Canvas, GLEventListener {
             gl.glDepthFunc(GL2.GL_LEQUAL); // Set to less equal to allow last drawn object to be on the top.
 
             if ((antiAliasingLevel > 0) && (antiAliasingLevel < 5) && (drawingTools.getGLCapacity().isAccumulationBufferPresent())) {
+                org.scilab.forge.scirenderer.renderer.Renderer renderer = rendererManager.createRenderer();
+                renderer.setDrawer(mainDrawer);
+
                 double[][] jitter = ANTI_ALIASING_JITTER[antiAliasingLevel];
                 Dimension dimension = getDimension();
                 gl.glClear(GL2.GL_ACCUM_BUFFER_BIT);
@@ -294,11 +282,14 @@ public final class JoGLCanvas implements Canvas, GLEventListener {
                     gl.glTranslated(aJitter[0] / dimension.getWidth(), aJitter[1] / dimension.getHeight(), 0.);
 
                     gl.glClear(GL2.GL_DEPTH_BUFFER_BIT);
-                    mainDrawer.draw(drawingTools);
+                    rendererManager.draw(drawingTools, renderer);
+                    //mainDrawer.draw(drawingTools);
                     gl.glReadBuffer(GL2.GL_BACK);
                     gl.glAccum(GL2.GL_ACCUM, 1f / jitter.length);
                 }
 
+                rendererManager.dispose(drawingTools, renderer);
+
                 gl.glDrawBuffer(GL2.GL_BACK);
                 gl.glAccum(GL2.GL_RETURN, 1.0f);
             } else {
@@ -347,6 +338,8 @@ public final class JoGLCanvas implements Canvas, GLEventListener {
         public void finalize() throws Throwable {
             running = false;
             semaphore.release();
+	    thread.interrupt();
+	    autoDrawable.destroy();
             super.finalize();
         }
 
@@ -368,15 +361,22 @@ public final class JoGLCanvas implements Canvas, GLEventListener {
 
         @Override
         public void run() {
-            while(running) {
+            while (running) {
                 try {
                     semaphore.acquire();
                     semaphore.drainPermits();
                     autoDrawable.display();
                 } catch (InterruptedException e) {
-                    Thread.currentThread().interrupt();
+                    if (running) {
+			Thread.currentThread().interrupt();
+		    }
                     break;
-                }
+                } catch (GLException e) {
+		    if (running) {
+			throw e;
+		    }
+                    break;
+		}
             }
         }
     }
