@@ -11,7 +11,9 @@
 
 package org.scilab.forge.scirenderer.implementation.jogl.texture;
 
-import com.jogamp.opengl.util.texture.TextureData;
+import com.jogamp.opengl.util.awt.TextureRenderer;
+import com.jogamp.opengl.util.texture.TextureCoords;
+
 import org.scilab.forge.scirenderer.SciRendererException;
 import org.scilab.forge.scirenderer.buffers.ElementsBuffer;
 import org.scilab.forge.scirenderer.implementation.jogl.JoGLCanvas;
@@ -21,12 +23,13 @@ import org.scilab.forge.scirenderer.texture.AnchorPosition;
 import org.scilab.forge.scirenderer.texture.Texture;
 import org.scilab.forge.scirenderer.texture.TextureManager;
 import org.scilab.forge.scirenderer.tranformations.Transformation;
-import org.scilab.forge.scirenderer.tranformations.TransformationFactory;
 import org.scilab.forge.scirenderer.tranformations.TransformationManager;
 import org.scilab.forge.scirenderer.tranformations.Vector3d;
 
 import javax.media.opengl.GL2;
+import java.awt.AlphaComposite;
 import java.awt.Dimension;
+import java.awt.Graphics2D;
 import java.nio.FloatBuffer;
 import java.util.Collection;
 import java.util.HashSet;
@@ -37,8 +40,10 @@ import java.util.Set;
  */
 public class JoGLTextureManager implements TextureManager {
     private final Set<JoGLTexture> allTextures = new HashSet<JoGLTexture>();
+    JoGLCanvas canvas;
 
     public JoGLTextureManager(JoGLCanvas canvas) {
+        this.canvas = canvas;
     }
 
     /**
@@ -62,7 +67,10 @@ public class JoGLTextureManager implements TextureManager {
      */
     public void draw(JoGLDrawingTools drawingTools, Texture texture) throws SciRendererException {
         if ((texture instanceof JoGLTexture) && (allTextures.contains((JoGLTexture) texture))) {
-            ((JoGLTexture) texture).draw(drawingTools);
+            final JoGLTexture jt = (JoGLTexture) texture;
+            jt.preDraw(drawingTools);
+            jt.draw(drawingTools);
+            jt.postDraw(drawingTools);
         }
     }
 
@@ -73,10 +81,13 @@ public class JoGLTextureManager implements TextureManager {
                 if (data != null) {
                     data.rewind();
                     float[] position = {0, 0, 0, 1};
+                    final JoGLTexture jt = (JoGLTexture) texture;
+                    jt.preDraw(drawingTools);
                     while (data.remaining() >= 4) {
                         data.get(position);
-                        ((JoGLTexture) texture).draw(drawingTools, anchor, new Vector3d(position), rotationAngle);
+                        jt.draw(drawingTools, anchor, new Vector3d(position), rotationAngle);
                     }
+                    jt.postDraw(drawingTools);
                 }
             }
         }
@@ -84,7 +95,10 @@ public class JoGLTextureManager implements TextureManager {
 
     public void draw(JoGLDrawingTools drawingTools, Texture texture, AnchorPosition anchor, Vector3d position, double rotationAngle) throws SciRendererException {
         if ((texture instanceof JoGLTexture) && (allTextures.contains((JoGLTexture) texture))) {
-            ((JoGLTexture) texture).draw(drawingTools, anchor, position, rotationAngle);
+            final JoGLTexture jt = (JoGLTexture) texture;
+            jt.preDraw(drawingTools);
+            jt.draw(drawingTools, anchor, position, rotationAngle);
+            jt.postDraw(drawingTools);
         }
     }
 
@@ -119,10 +133,12 @@ public class JoGLTextureManager implements TextureManager {
     /**
      * Inner class for {@link Texture} implementation.
      */
-    private class JoGLTexture extends AbstractTexture implements Texture {
+    public class JoGLTexture extends AbstractTexture implements Texture {
         private com.jogamp.opengl.util.texture.Texture[] textures;
         private int wCuts;
         private int hCuts;
+        private double sfactor = 1;
+        private double tfactor = 1;
 
         /**
          * Default constructor.
@@ -142,11 +158,17 @@ public class JoGLTextureManager implements TextureManager {
                 if (textures.length == 1) {
                     gl.glEnable(GL2.GL_TEXTURE_2D);
                     textures[0].setTexParameteri(gl, GL2.GL_TEXTURE_MAG_FILTER, getAsGLFilter(getMagnificationFilter(), false));
-                    textures[0].setTexParameteri(gl, GL2.GL_TEXTURE_MIN_FILTER, getAsGLFilter(getMinifyingFilter(), true));
+                    textures[0].setTexParameteri(gl, GL2.GL_TEXTURE_MIN_FILTER, getAsGLFilter(getMinifyingFilter(), false));
                     textures[0].setTexParameteri(gl, GL2.GL_TEXTURE_WRAP_S, getAsGLWrappingMode(getSWrappingMode()));
                     textures[0].setTexParameteri(gl, GL2.GL_TEXTURE_WRAP_T, getAsGLWrappingMode(getTWrappingMode()));
                     gl.glTexEnvi(GL2.GL_TEXTURE_ENV, GL2.GL_TEXTURE_ENV_MODE, GL2.GL_REPLACE);
                     textures[0].bind(gl);
+
+                    /* sfactor and tfactor are useful to have the correct texture coordinates when the texture
+                       was transformed into a power-of-two texture */
+                    Dimension textureSize = getDataProvider().getTextureSize();
+                    sfactor = (double) textureSize.width / (double) textures[0].getWidth();
+                    tfactor = (double) textureSize.height / (double) textures[0].getHeight();
                 } else {
                     throw new SciRendererException("Texture is too large");
                 }
@@ -162,9 +184,7 @@ public class JoGLTextureManager implements TextureManager {
          */
         private void checkData(JoGLDrawingTools drawingTools) throws SciRendererException {
             if (isValid() && !upToDate) {
-                upToDate = true;
-
-		GL2 gl = drawingTools.getGl().getGL2();
+                GL2 gl = drawingTools.getGl().getGL2();
                 releaseTextures(gl);
 
                 Dimension textureSize = getDataProvider().getTextureSize();
@@ -176,30 +196,23 @@ public class JoGLTextureManager implements TextureManager {
                 int k = 0;
                 for (int i = 0; i < wCuts; i++) {
                     for (int j = 0; j < hCuts; j++) {
-                        textures[k] = com.jogamp.opengl.util.texture.TextureIO.newTexture(GL2.GL_TEXTURE_2D);
                         int x = i * maxSize;
                         int y = j * maxSize;
                         int width = getSubTextureSize((i == (wCuts - 1)), textureSize.width, maxSize);
                         int height = getSubTextureSize((j == (hCuts - 1)), textureSize.height, maxSize);
 
-                        TextureData data = new TextureData(
-			        gl.getGLProfile(),
-                                GL2.GL_RGBA,
-                                width,
-                                height,
-                                0, // no border
-                                GL2.GL_RGBA, // pixel format.
-                                GL2.GL_UNSIGNED_BYTE, // pixel type.
-                                true, // mipmap
-                                false, // no compression
-                                false, // no vertical flip needed TODO as an option from data provider.
-                                getDataProvider().getSubData(x, y, width, height),
-                                null   // no flusher
-                        );
-                        textures[k].updateImage(gl, data);
+                        TextureRenderer renderer = new TextureRenderer(width, height, true, false);
+                        Graphics2D g2d = renderer.createGraphics();
+                        g2d.setComposite(AlphaComposite.Src);
+                        g2d.drawImage(getDataProvider().getSubImage(x, y, width, height), null, 0, 0);
+                        g2d.dispose();
+                        renderer.markDirty(0, 0, width, height);
+                        textures[k] = renderer.getTexture();
+
                         k++;
                     }
                 }
+                upToDate = true;
             }
         }
 
@@ -212,46 +225,85 @@ public class JoGLTextureManager implements TextureManager {
             }
         }
 
+        public void preDraw(JoGLDrawingTools drawingTools) throws SciRendererException {
+            checkData(drawingTools);
+            final GL2 gl = drawingTools.getGl().getGL2();
+
+            gl.glMatrixMode(GL2.GL_TEXTURE);
+            gl.glPushMatrix();
+            gl.glLoadIdentity();
+
+            gl.glMatrixMode(GL2.GL_PROJECTION);
+            gl.glPushMatrix();
+            gl.glLoadIdentity();
+
+            gl.glEnable(GL2.GL_TEXTURE_2D);
+
+            gl.glEnable(GL2.GL_BLEND);
+            gl.glBlendFunc(GL2.GL_ONE, GL2.GL_ONE_MINUS_SRC_ALPHA);
+
+            gl.glEnable(GL2.GL_ALPHA_TEST);
+            gl.glAlphaFunc(GL2.GL_GREATER, 0.0f);
+
+            gl.glPushAttrib(GL2.GL_ALL_ATTRIB_BITS);
+
+            gl.glTexEnvi(GL2.GL_TEXTURE_ENV, GL2.GL_TEXTURE_ENV_MODE, GL2.GL_REPLACE);
+
+            for (int k = 0; k < wCuts * hCuts; k++) {
+                textures[k].enable(gl);
+                textures[k].bind(gl);
+            }
+        }
+
         public void draw(JoGLDrawingTools drawingTools, AnchorPosition anchor, Vector3d position, double rotationAngle) throws SciRendererException {
             TransformationManager transformationManager = drawingTools.getTransformationManager();
             Transformation canvasProjection = transformationManager.getCanvasProjection();
-
             boolean sceneCoordinate = drawingTools.getTransformationManager().isUsingSceneCoordinate();
-            if (!sceneCoordinate) {
-                drawingTools.getTransformationManager().useSceneCoordinate();
-            }
-
+            Dimension dimension = drawingTools.getCanvas().getDimension();
             Vector3d projected;
+            final GL2 gl = drawingTools.getGl().getGL2();
+
             if (sceneCoordinate) {
                 projected = canvasProjection.project(position);
             } else {
                 projected = position;
             }
 
-            Dimension dimension = drawingTools.getCanvas().getDimension();
+            gl.glMatrixMode(GL2.GL_MODELVIEW);
+            gl.glPushMatrix();
+            gl.glLoadIdentity();
+            gl.glOrtho(0, dimension.width, 0,  dimension.height, 1, -1);
 
-            Vector3d s = new Vector3d(2 / dimension.getWidth(), 2 / dimension.getHeight(), 1);
-            Vector3d t = new Vector3d(
-                    (projected.getX() * 2) / dimension.getWidth() - 1,
-                    (projected.getY() * 2) / dimension.getHeight() - 1,
-                    projected.getZ()
-            );
-
-            Transformation translation = TransformationFactory.getTranslateTransformation(getAnchorDeltaX(anchor), getAnchorDeltaY(anchor), 0);
-
-            transformationManager.getProjectionStack().push(TransformationFactory.getIdentity());
-            transformationManager.getModelViewStack().push(TransformationFactory.getAffineTransformation(s, t));
-            transformationManager.getModelViewStack().pushRightMultiply(TransformationFactory.getRotationTransformation(-rotationAngle, 0, 0, 1));
-            transformationManager.getModelViewStack().pushRightMultiply(translation);
-            draw(drawingTools);
-            transformationManager.getModelViewStack().pop();
-            transformationManager.getModelViewStack().pop();
-            transformationManager.getModelViewStack().pop();
-            transformationManager.getProjectionStack().pop();
-
-            if (!sceneCoordinate) {
-                drawingTools.getTransformationManager().useWindowCoordinate();
+            if (rotationAngle == 0) {
+                gl.glTranslated(Math.round(projected.getX() + getAnchorDeltaX(anchor)), Math.round(projected.getY() + getAnchorDeltaY(anchor)), projected.getZ());
+            } else {
+                gl.glTranslated(Math.round(projected.getX()), Math.round(projected.getY()), projected.getZ());
+                gl.glRotated(rotationAngle, 0, 0, 1);
+                gl.glTranslated(Math.round(getAnchorDeltaX(anchor)), Math.round(getAnchorDeltaY(anchor)), 0);
             }
+
+            draw(drawingTools);
+
+            gl.glMatrixMode(GL2.GL_MODELVIEW);
+            gl.glPopMatrix();
+        }
+
+        public void postDraw(JoGLDrawingTools drawingTools) {
+            final GL2 gl = drawingTools.getGl().getGL2();
+
+            for (int k = 0; k < wCuts * hCuts; k++) {
+                textures[k].disable(gl);
+            }
+
+            gl.glPopAttrib();
+            gl.glDisable(GL2.GL_ALPHA_TEST);
+            gl.glDisable(GL2.GL_TEXTURE_2D);
+            gl.glDisable(GL2.GL_BLEND);
+
+            gl.glMatrixMode(GL2.GL_PROJECTION);
+            gl.glPopMatrix();
+            gl.glMatrixMode(GL2.GL_TEXTURE);
+            gl.glPopMatrix();
         }
 
         /**
@@ -260,53 +312,45 @@ public class JoGLTextureManager implements TextureManager {
          * @throws SciRendererException if the texture is invalid.
          */
         public void draw(JoGLDrawingTools drawingTools) throws SciRendererException {
-            checkData(drawingTools);
             final int maxSize = drawingTools.getGLCapacity().getMaximumTextureSize();
             final Dimension textureSize = getDataProvider().getTextureSize();
             final GL2 gl = drawingTools.getGl().getGL2();
-            gl.glEnable(GL2.GL_TEXTURE_2D);
-
-            gl.glEnable(GL2.GL_BLEND);
-            gl.glBlendFunc(GL2.GL_SRC_ALPHA, GL2.GL_ONE_MINUS_SRC_ALPHA);
-
-            gl.glEnable(GL2.GL_ALPHA_TEST);
-            gl.glAlphaFunc(GL2.GL_GREATER, 0.0f);
 
             int k = 0;
             for (int i = 0; i < wCuts; i++) {
                 for (int j = 0; j < hCuts; j++) {
-                    int x = i * maxSize;
-                    int y = j * maxSize;
-                    int width = getSubTextureSize((i == (wCuts - 1)), textureSize.width, maxSize);
-                    int height = getSubTextureSize((j == (hCuts - 1)), textureSize.height, maxSize);
+                    final int x = i * maxSize;
+                    final int y = j * maxSize;
+                    final int width = getSubTextureSize((i == (wCuts - 1)), textureSize.width, maxSize);
+                    final int height = getSubTextureSize((j == (hCuts - 1)), textureSize.height, maxSize);
 
-                    textures[k].setTexParameteri(gl, GL2.GL_TEXTURE_MAG_FILTER, getAsGLFilter(getMagnificationFilter(), false));
-                    textures[k].setTexParameteri(gl, GL2.GL_TEXTURE_MIN_FILTER, getAsGLFilter(getMinifyingFilter(), true));
-                    textures[k].setTexParameteri(gl, GL2.GL_TEXTURE_WRAP_S, getAsGLWrappingMode(getSWrappingMode()));
-                    textures[k].setTexParameteri(gl, GL2.GL_TEXTURE_WRAP_T, getAsGLWrappingMode(getTWrappingMode()));
-                    gl.glTexEnvi(GL2.GL_TEXTURE_ENV, GL2.GL_TEXTURE_ENV_MODE, GL2.GL_REPLACE);
-                    textures[k].bind(gl);
+                    // if the texture has been transformed in a power-of-two texture we need to have the correct coords.
+                    final TextureCoords coords = textures[k].getSubImageTexCoords(0, 0, width, height);
 
                     gl.glBegin(GL2.GL_QUADS);
-                    gl.glTexCoord2d(0, 1);
+                    gl.glTexCoord2f(coords.left(), coords.bottom());
                     gl.glVertex2f(x, y);
-
-                    gl.glTexCoord2d(1, 1);
+                    gl.glTexCoord2f(coords.right(), coords.bottom());
                     gl.glVertex2f(x + width, y);
-
-                    gl.glTexCoord2d(1, 0);
+                    gl.glTexCoord2f(coords.right(), coords.top());
                     gl.glVertex2f(x + width, y + height);
-
-                    gl.glTexCoord2d(0, 0);
+                    gl.glTexCoord2f(coords.left(), coords.top());
                     gl.glVertex2f(x, y + height);
                     gl.glEnd();
 
                     k++;
                 }
             }
-            gl.glDisable(GL2.GL_ALPHA_TEST);
-            gl.glDisable(GL2.GL_TEXTURE_2D);
-            gl.glDisable(GL2.GL_BLEND);
+        }
+
+        @Override
+        public double getSScaleFactor() {
+            return sfactor;
+        }
+
+        @Override
+        public double getTScaleFactor() {
+            return tfactor;
         }
 
         /**
@@ -331,12 +375,12 @@ public class JoGLTextureManager implements TextureManager {
 
         private int getAsGLWrappingMode(Texture.Wrap wrappingMode) {
             switch (wrappingMode) {
-                case CLAMP:
-                    return GL2.GL_CLAMP;
-                case REPEAT:
-                    return GL2.GL_REPEAT;
-                default:
-                    return GL2.GL_REPEAT;
+            case CLAMP:
+                return GL2.GL_CLAMP;
+            case REPEAT:
+                return GL2.GL_REPEAT;
+            default:
+                return GL2.GL_REPEAT;
             }
         }
 
@@ -344,27 +388,27 @@ public class JoGLTextureManager implements TextureManager {
             int returnedValue;
             if (mipmap) {
                 switch (filter) {
-                    case LINEAR:
-                        returnedValue = GL2.GL_LINEAR_MIPMAP_LINEAR;
-                        break;
-                    case NEAREST:
-                        returnedValue = GL2.GL_NEAREST_MIPMAP_NEAREST;
-                        break;
-                    default:
-                        returnedValue = GL2.GL_NEAREST;
-                        break;
+                case LINEAR:
+                    returnedValue = GL2.GL_LINEAR_MIPMAP_LINEAR;
+                    break;
+                case NEAREST:
+                    returnedValue = GL2.GL_NEAREST_MIPMAP_NEAREST;
+                    break;
+                default:
+                    returnedValue = GL2.GL_NEAREST;
+                    break;
                 }
             } else {
                 switch (filter) {
-                    case LINEAR:
-                        returnedValue = GL2.GL_LINEAR;
-                        break;
-                    case NEAREST:
-                        returnedValue = GL2.GL_NEAREST;
-                        break;
-                    default:
-                        returnedValue = GL2.GL_NEAREST;
-                        break;
+                case LINEAR:
+                    returnedValue = GL2.GL_LINEAR;
+                    break;
+                case NEAREST:
+                    returnedValue = GL2.GL_NEAREST;
+                    break;
+                default:
+                    returnedValue = GL2.GL_NEAREST;
+                    break;
                 }
             }
             return returnedValue;
@@ -381,23 +425,23 @@ public class JoGLTextureManager implements TextureManager {
          * @param anchor the given anchor.
          * @return the deltaX to apply to the sprite in regards to the given anchor.
          */
-        protected float getAnchorDeltaX(AnchorPosition anchor) {
+        protected double getAnchorDeltaX(AnchorPosition anchor) {
             int spriteWidth = getDataProvider().getTextureSize().width;
             switch (anchor) {
-                case LEFT:
-                case LOWER_LEFT:
-                case UPPER_LEFT:
-                    return 0;
-                case UP:
-                case CENTER:
-                case DOWN:
-                    return -spriteWidth / 2f;
-                case RIGHT:
-                case LOWER_RIGHT:
-                case UPPER_RIGHT:
-                    return -spriteWidth;
-                default:
-                    return 0;
+            case LEFT:
+            case LOWER_LEFT:
+            case UPPER_LEFT:
+                return 0;
+            case UP:
+            case CENTER:
+            case DOWN:
+                return -spriteWidth / 2f;
+            case RIGHT:
+            case LOWER_RIGHT:
+            case UPPER_RIGHT:
+                return -spriteWidth;
+            default:
+                return 0;
             }
         }
 
@@ -406,23 +450,23 @@ public class JoGLTextureManager implements TextureManager {
          * @param anchor the given anchor.
          * @return the deltaY to apply to the sprite in regards to the given anchor.
          */
-        protected float getAnchorDeltaY(AnchorPosition anchor) {
+        protected double getAnchorDeltaY(AnchorPosition anchor) {
             int spriteHeight = getDataProvider().getTextureSize().height;
             switch (anchor) {
-                case UPPER_LEFT:
-                case UP:
-                case UPPER_RIGHT:
-                    return -spriteHeight;
-                case LEFT:
-                case CENTER:
-                case RIGHT:
-                    return -spriteHeight / 2f;
-                case LOWER_LEFT:
-                case DOWN:
-                case LOWER_RIGHT:
-                    return 0;
-                default:
-                    return 0;
+            case UPPER_LEFT:
+            case UP:
+            case UPPER_RIGHT:
+                return -spriteHeight;
+            case LEFT:
+            case CENTER:
+            case RIGHT:
+                return -spriteHeight / 2f;
+            case LOWER_LEFT:
+            case DOWN:
+            case LOWER_RIGHT:
+                return 0;
+            default:
+                return 0;
             }
         }
     }
