@@ -1,7 +1,8 @@
 /*
  * Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
  * Copyright (C) 2009-2012 - DIGITEO - Pierre Lando
- *
+ * Copyright (C) 2012 - Scilab Enterprises - Bruno JOFRET
+ *  *
  * This file must be used under the terms of the CeCILL.
  * This source file is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -11,6 +12,17 @@
 
 package org.scilab.forge.scirenderer.ruler;
 
+import java.awt.Dimension;
+import java.awt.geom.Rectangle2D;
+import java.nio.FloatBuffer;
+import java.text.DecimalFormat;
+import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.LinkedList;
+import java.util.List;
+import java.util.Map;
+import java.util.concurrent.ConcurrentHashMap;
+
 import org.scilab.forge.scirenderer.DrawingTools;
 import org.scilab.forge.scirenderer.SciRendererException;
 import org.scilab.forge.scirenderer.buffers.BuffersManager;
@@ -25,14 +37,6 @@ import org.scilab.forge.scirenderer.texture.TextureManager;
 import org.scilab.forge.scirenderer.tranformations.Transformation;
 import org.scilab.forge.scirenderer.tranformations.Vector3d;
 
-import java.awt.geom.Rectangle2D;
-import java.nio.FloatBuffer;
-import java.text.DecimalFormat;
-import java.util.LinkedList;
-import java.util.List;
-import java.util.Map;
-import java.util.concurrent.ConcurrentHashMap;
-
 /**
  * @author Pierre Lando
  */
@@ -53,6 +57,8 @@ public class RulerDrawer {
      */
     private RulerSpriteFactory spriteFactory;
 
+    private OneShotRulerDrawer oneShotRulerDrawer;
+    
     /**
      * Ruler drawer constructor.
      * @param textureManager the {@link TextureManager} of the canvas where the ruler will be drawn.
@@ -60,6 +66,7 @@ public class RulerDrawer {
     public RulerDrawer(TextureManager textureManager) {
         this.textureManager = textureManager;
         this.spriteFactory = new DefaultRulerSpriteFactory();
+        this.oneShotRulerDrawer = new OneShotRulerDrawer();
     }
 
     /**
@@ -69,7 +76,7 @@ public class RulerDrawer {
      * @return the {@link RulerDrawingResult} give information about how the ruler have been drawn.
      */
     public RulerDrawingResult draw(DrawingTools drawingTools, RulerModel model) {
-        return new OneShotRulerDrawer(drawingTools, model).getDrawingResult();
+        return oneShotRulerDrawer.drawWithResults(drawingTools, model);
     }
 
     /**
@@ -97,13 +104,17 @@ public class RulerDrawer {
      */
     private class OneShotRulerDrawer {
 
-        private final RulerDrawingResult rulerDrawingResult;
-        private final Transformation canvasProjection;
-        private final RulerModel rulerModel;
-        private final Vector3d windowSubTicksDelta;
-        private final Vector3d windowTicksDelta;
+        private Transformation canvasProjection;
+        private RulerModel rulerModel;
+        private Vector3d windowSubTicksDelta;
+        private Vector3d windowTicksDelta;
 
-        private final List<PositionedSprite> spritesList = new LinkedList<PositionedSprite>();
+        /**
+         * Sprite Dimension map. Used as cached values in order not to charge DataProvider.
+         */
+        private Map<Double, Dimension> spriteDimensionMap = new HashMap<Double, Dimension>();
+        
+        private List<PositionedSprite> spritesList = new LinkedList<PositionedSprite>();
 
         /**
          * The maximum distance corresponding to the actually displayed sprites.
@@ -119,13 +130,19 @@ public class RulerDrawer {
         private List<Double> ticksValue;
         private int density;
 
+        public OneShotRulerDrawer() { }
+        
         /**
          * Constructor.
          * @param drawingTools the {@link DrawingTools} of the canvas where the ruler will be drawn.
          * @param rulerModel the {@link RulerModel} of the drawn ruler.
          */
-        public OneShotRulerDrawer(DrawingTools drawingTools, RulerModel rulerModel) {
+        public RulerDrawingResult drawWithResults(DrawingTools drawingTools, RulerModel rulerModel) {
             this.rulerModel = rulerModel;
+            subTicksValue = new LinkedList<Double>();
+            ticksValue = new LinkedList<Double>();
+            spritesList = new LinkedList<PositionedSprite>();
+            
             canvasProjection = drawingTools.getTransformationManager().getCanvasProjection();
 
             Vector3d windowTicksDirection = canvasProjection.projectDirection(rulerModel.getTicksDirection());
@@ -137,23 +154,16 @@ public class RulerDrawer {
 
             DecimalFormat format;
             if (rulerModel.isAutoTicks()) {
-                format = computeAutoGraduation();
+            	format = computeAutoGraduation();
             } else {
                 format = computeUserGraduation();
             }
             computeTicksData();
-            draw(drawingTools);
+        	draw(drawingTools);
+            
             double distRatio = computeTicksDistanceRatio(windowTicksDirection.getNorm());
 
-            rulerDrawingResult = new RulerDrawingResult(format, ticksValue, subTicksValue, density, distRatio, normalizedProjectedTicksDirection);
-        }
-
-        /**
-         * Return the drawing result.
-         * @return the drawing result.
-         */
-        public RulerDrawingResult getDrawingResult() {
-            return rulerDrawingResult;
+            return new RulerDrawingResult(format, ticksValue, subTicksValue, density, distRatio, normalizedProjectedTicksDirection);
         }
 
         /**
@@ -214,23 +224,27 @@ public class RulerDrawer {
             Graduations ticksGraduation = currentGraduations;
             DecimalFormat format = currentGraduations.getFormat();
             boolean canGetMore = true;
+            List<PositionedSprite> newSpritesList = new ArrayList<PositionedSprite>();
             while (currentGraduations != null) {
                 /* The maximum distance to any of the sprites' farthest sides at a given iteration. */
                 double currentMaximalSpritesDistance = 0;
 
-                List<PositionedSprite> newSpritesList = new LinkedList<PositionedSprite>();
+                newSpritesList.clear();
                 List<Double> ticks = currentGraduations.getNewValues();
                 for (double value : ticks) {
                     Texture sprite = computeSprite(value, format);
                     Vector3d windowPosition = canvasProjection.project(rulerModel.getPosition(value));
-
-                    Vector3d delta = projectCenterToEdge(sprite, windowTicksDelta);
-                    newSpritesList.add(new PositionedSprite(sprite, windowPosition.plus(windowTicksDelta.plus(delta))));
+                    
+                    Dimension textureSize = computeSpriteDimension(value);
+                    
+                    Vector3d delta = projectCenterToEdge(textureSize, windowTicksDelta);
+                    PositionedSprite newSprite = new PositionedSprite(sprite, textureSize, windowPosition.plus(windowTicksDelta.plus(delta)));
+                    newSpritesList.add(newSprite);
 
                     Vector3d farDelta = windowTicksDelta.plus(delta.times(2.0));
                     currentMaximalSpritesDistance = Math.max(currentMaximalSpritesDistance, farDelta.getNorm());
                 }
-
+                
                 if (collide(newSpritesList, rulerModel.getMargin()) || collide(spritesList, newSpritesList, rulerModel.getMargin())) {
                     currentGraduations = currentGraduations.getAlternative();
                     canGetMore = false;
@@ -246,8 +260,10 @@ public class RulerDrawer {
                 }
             }
 
+            
             this.graduations = ticksGraduation;
             this.maximalSpritesDistance = maxSpritesDistance;
+
             return format;
         }
 
@@ -297,11 +313,11 @@ public class RulerDrawer {
                 if (subGraduation != null) {
                     subTicksValue = subGraduation.getAllValues();
                 } else {
-                    subTicksValue = new LinkedList<Double>();
+                    subTicksValue = new ArrayList<Double>();
                 }
             } else {
-                subTicksValue = new LinkedList<Double>();
-                ticksValue = new LinkedList<Double>();
+                subTicksValue = new ArrayList<Double>();
+                ticksValue = new ArrayList<Double>();
                 density = 0;
             }
         }
@@ -394,12 +410,35 @@ public class RulerDrawer {
             }
 
             /* +1 is used to have a space between the tick and its label */
-            double ratioX = sprite.getDataProvider().getTextureSize().width / Math.abs(usedDirection.getX()) + 1;
-            double ratioY = sprite.getDataProvider().getTextureSize().height / Math.abs(usedDirection.getY()) + 1;
+            Dimension textureSize = sprite.getDataProvider().getTextureSize();
+            double ratioX = textureSize.width / Math.abs(usedDirection.getX()) + 1;
+            double ratioY = textureSize.height / Math.abs(usedDirection.getY()) + 1;
             double ratio = Math.min(ratioY, ratioX) / 2;
             return usedDirection.times(ratio);
         }
 
+        /**
+         * Compute and return a translation along the given <code>direction</code>.
+         * This translation is the vector between the <code>sprite</code> center and is projection along the given
+         * {@see direction} to the sprite edges.
+         * @param textureSize the size of corresponding texture
+         * @param direction the given direction.
+         * @return the vector between the sprite center and is projection to the sprite edges.
+         */
+        private Vector3d projectCenterToEdge(Dimension textureSize, Vector3d direction) {
+            Vector3d usedDirection;
+            if ((direction == null) || (direction.isZero())) {
+                usedDirection = new Vector3d(1, 0, 0);
+            } else {
+                usedDirection = direction;
+            }
+
+            /* +1 is used to have a space between the tick and its label */
+            double ratioX = textureSize.width / Math.abs(usedDirection.getX()) + 1;
+            double ratioY = textureSize.height / Math.abs(usedDirection.getY()) + 1;
+            double ratio = Math.min(ratioY, ratioX) / 2;
+            return usedDirection.times(ratio);
+        }
 
         /**
          * Fill a vertices buffer with the needed data to draw a ruler.
@@ -461,6 +500,17 @@ public class RulerDrawer {
             return sprite;
         }
 
+        private Dimension computeSpriteDimension(double value)
+        {
+        	Dimension spriteDimension = spriteDimensionMap.get(value);
+        	if (spriteDimension == null) {
+        		spriteDimension = spriteMap.get(value).getDataProvider().getTextureSize();
+        		spriteDimensionMap.put(value, spriteDimension);
+        	}
+        	
+        	return spriteDimension;
+        }
+        
         /**
          * This class is a basic container for a {@link Texture} and an associated window position.
          *
@@ -478,17 +528,35 @@ public class RulerDrawer {
              * @param windowPosition the window position.
              */
             public PositionedSprite(Texture sprite,  Vector3d windowPosition) {
-                this.windowPosition = windowPosition;
+            	//long tic = Calendar.getInstance().getTimeInMillis();
+            	this.windowPosition = windowPosition;
+                this.sprite = sprite;
+
+                Dimension textureSize = sprite.getDataProvider().getTextureSize();
+                windowsBounds = new Rectangle2D.Double(
+                        windowPosition.getX(),
+                        windowPosition.getY(),
+                        textureSize.width,
+                        textureSize.height
+                );
+                
+                //System.err.println("====[new PositionedSprite] time = "+(Calendar.getInstance().getTimeInMillis() - tic));
+            }
+
+            public PositionedSprite(Texture sprite, Dimension textureSize, Vector3d windowPosition) {
+            	//long tic = Calendar.getInstance().getTimeInMillis();
+            	this.windowPosition = windowPosition;
                 this.sprite = sprite;
 
                 windowsBounds = new Rectangle2D.Double(
                         windowPosition.getX(),
                         windowPosition.getY(),
-                        sprite.getDataProvider().getTextureSize().width,
-                        sprite.getDataProvider().getTextureSize().height
+                        textureSize.width,
+                        textureSize.height
                 );
+                
+                //System.err.println("====[new PositionedSprite] time = "+(Calendar.getInstance().getTimeInMillis() - tic));
             }
-
             /**
              * Return the {@link Texture}
              * @return the {@link Texture}
