@@ -119,4 +119,14 @@ public class AbstractTexture implements Texture {
     public void dataUpdated() {
         upToDate = false;
     }
+
+    @Override
+    public double getSScaleFactor() {
+        return 1;
+    }
+
+    @Override
+    public double getTScaleFactor() {
+        return 1;
+    }
 }
